@@ -23,6 +23,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <signal.h>
 #include <readline/history.h>
 #include <readline/readline.h>
 #include "dslib.h"
@@ -39,7 +42,7 @@
 #define FLOAT_WIDTH 40
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 #define MAX_BITS 128
-#define _ALIGNMENT_MASK 0xF
+#define ALIGNMENT_MASK_4BIT 0xF
 
 typedef unsigned char bool;
 typedef unsigned char uchar;
@@ -71,7 +74,7 @@ typedef struct {
 	uchar loglvl  : 2;
 } settings;
 
-static char *VERSION = "2.2";
+static char *VERSION = "2.3";
 static char *units[] = {"b", "kib", "mib", "gib", "tib", "kb", "mb", "gb", "tb"};
 static char *logarr[] = {"ERROR", "WARNING", "INFO", "DEBUG"};
 
@@ -118,7 +121,7 @@ static size_t bstrlcpy(char *dest, const char *src, size_t n)
 	static ulong *s, *d;
 	static size_t len, blocks;
 	static const uint lsize = sizeof(ulong);
-	static const uint _WSHIFT = (sizeof(ulong) == 8) ? 3 : 2;
+	static const uint WORD_SHIFT = (sizeof(ulong) == 8) ? 3 : 2;
 
 	if (!src || !dest || !n)
 		return 0;
@@ -134,10 +137,11 @@ static size_t bstrlcpy(char *dest, const char *src, size_t n)
 	 * To enable -O3 ensure src and dest are 16-byte aligned
 	 * More info: http://www.felixcloutier.com/x86/MOVDQA.html
 	 */
-	if ((n >= lsize) && (((ulong)src & _ALIGNMENT_MASK) == 0 && ((ulong)dest & _ALIGNMENT_MASK) == 0)) {
+	if ((n >= lsize) && (((ulong)src & ALIGNMENT_MASK_4BIT) == 0
+	    && ((ulong)dest & ALIGNMENT_MASK_4BIT) == 0)) {
 		s = (ulong *)src;
 		d = (ulong *)dest;
-		blocks = n >> _WSHIFT;
+		blocks = n >> WORD_SHIFT;
 		n &= lsize - 1;
 
 		while (blocks) {
@@ -186,6 +190,9 @@ static int try_bc(char *expr)
 
 	log(DEBUG, "expression: \"%s\"\n", expr);
 
+	if (!strcmp(expr, "quit") || !strcmp(expr, "exit"))
+		exit(0);
+
 	if (pipe(pipe_pc) == -1 || pipe(pipe_cp) == -1) {
 		log(ERROR, "pipe()!\n");
 		exit(EXIT_FAILURE);
@@ -208,7 +215,7 @@ static int try_bc(char *expr)
 		dup2(pipe_cp[1], STDOUT_FILENO); // Give stdout to parent
 		dup2(pipe_cp[1], STDERR_FILENO); // Give stderr to parent
 
-		ret = execlp(ptr, ptr, (char*) NULL);
+		int ret = execlp(ptr, ptr, (char*) NULL);
 		log(ERROR, "execlp() failed!\n");
 		exit(ret);
 	}
@@ -263,6 +270,17 @@ static int try_bc(char *expr)
 		exit(-1);
 	}
 
+	if (write(pipe_pc[1], "quit\n", 5) != 5) {
+		log(ERROR, "write(7)! [%s]\n", strerror(errno));
+		exit(-1);
+	}
+
+	if (cfg.calc)
+		kill(pid, SIGTERM);
+
+	close(pipe_cp[0]);
+	close(pipe_pc[1]);
+
 	buffer[ret] = '\0';
 
 	if (buffer[0] != '(') {
@@ -271,6 +289,9 @@ static int try_bc(char *expr)
 			++ptr;
 
 		printf("%s", ptr);
+		if (cfg.calc && strstr(ptr, "is undefined"))
+			return -1;
+
 		len = bstrlcpy(lastres.p, ptr, NUM_LEN);
 
 		/* remove newline appended at the end of result by bc */
@@ -302,22 +323,28 @@ static int try_bc(char *expr)
 static void binprint(maxuint_t n)
 {
 	int count = MAX_BITS - 1;
-	char binstr[MAX_BITS + 1] = {0};
+	int pos = MAX_BITS + (MAX_BITS >> 2) - 1;
+	char binstr[MAX_BITS + (MAX_BITS >> 2) + 1] = {0};
 
 	if (!n) {
-		printf("0b0");
+		printf("0");
 		return;
 	}
 
 	while (n && count >= 0) {
-		binstr[count] = "01"[n & 1];
-		--count;
+		binstr[pos] = "01"[n & 1];
+		--pos;
 		n >>= 1;
+		if (n && count && !(count & 7)) {
+			binstr[pos] = ' ';
+			--pos;
+		}
+		--count;
 	}
 
-	++count;
+	++pos;
 
-	printf("0b%s", binstr + count);
+	printf("%s", binstr + pos);
 }
 
 static char *getstr_u128(maxuint_t n, char *buf)
@@ -352,7 +379,7 @@ static char *getstr_f128(maxfloat_t val, char *buf)
 
 static void printval(maxfloat_t val, char *unit)
 {
-	if (val - (maxuint_t)val == 0)
+	if (val - (maxuint_t)val == 0) // NOLINT
 		printf("%40s %s\n", getstr_u128((maxuint_t)val, uint_buf), unit);
 	else
 		printf("%s %s\n", getstr_f128(val, float_buf), unit);
@@ -540,31 +567,31 @@ static maxuint_t convertbyte(char *buf, int *ret)
 	/* Convert and print in IEC standard units */
 
 	printf("\n            IEC standard (base 2)\n\n");
-	val = bytes / (maxfloat_t)1024;
+	val = (maxfloat_t)bytes / 1024;
 	printval(val, "KiB");
 
-	val = bytes / (maxfloat_t)(1 << 20);
+	val = (maxfloat_t)bytes / (1 << 20);
 	printval(val, "MiB");
 
-	val = bytes / (maxfloat_t)(1 << 30);
+	val = (maxfloat_t)bytes / (1 << 30);
 	printval(val, "GiB");
 
-	val = bytes / (maxfloat_t)((maxuint_t)1 << 40);
+	val = (maxfloat_t)bytes / ((unsigned long long)1 << 40);
 	printval(val, "TiB");
 
 	/* Convert and print in SI standard values */
 
 	printf("\n            SI standard (base 10)\n\n");
-	val = bytes / (maxfloat_t)1000;
+	val = (maxfloat_t)bytes / 1000;
 	printval(val, "kB");
 
-	val = bytes / (maxfloat_t)1000000;
+	val = (maxfloat_t)bytes / 1000000;
 	printval(val, "MB");
 
-	val = bytes / (maxfloat_t)1000000000;
+	val = (maxfloat_t)bytes / 1000000000;
 	printval(val, "GB");
 
-	val = bytes / (maxfloat_t)1000000000000;
+	val = (maxfloat_t)bytes / 1000000000000;
 	printval(val, "TB");
 
 	return bytes;
@@ -1021,9 +1048,8 @@ static bool chs2lba(char *chs, maxuint_t *lba)
 		return FALSE;
 	}
 
-
-	*lba = param[3] * param[4] * param[0]; /* MH * MS * C */
-	*lba += param[4] * param[1]; /* MS * H */
+	*lba = (maxuint_t)param[3] * param[4] * param[0]; /* MH * MS * C */
+	*lba += (maxuint_t)param[4] * param[1]; /* MS * H */
 
 	*lba += param[2] - 1; /* S - 1 */
 
@@ -1152,7 +1178,7 @@ Storage expression calculator.\n\n\
 positional arguments:\n\
  expr       expression in decimal/hex operands\n\
  N [unit]   capacity in B/KiB/MiB/GiB/TiB/kB/MB/GB/TB\n\
-            see https://wiki.ubuntu.com/UnitsPolicy\n\
+            https://en.wikipedia.org/wiki/Binary_prefix\n\
             default unit is B (byte), case is ignored\n\
             N can be decimal or '0x' prefixed hex value\n\n\
 optional arguments:\n\
@@ -2111,6 +2137,9 @@ int main(int argc, char **argv)
 
 		printf("q/double Enter -> quit, ? -> help\n");
 		while ((tmp = readline(prompt)) != NULL) {
+			if (!tmp)
+				exit(0);
+
 			/* Quit on double Enter */
 			if (tmp[0] == '\0') {
 				if (enters == 1) {
